# -*- coding: UTF-8 -*-
"""Définition et enregistrement des actions"""

import os
# imports ead
from ead2.backend.lib.error import SignatureError
from ead2.backend.config.help import help_templates
from ead2.backend.actions import tools
from ead2.lib.libead import uni

REGISTERED_ACTIONS = {}
REGISTERED_CATEGORIES = []


class TypedObject(object):
    """Type de base pour les paramètres des actions
    cette classe encapsule la valeur `value`
    et y affecte des contraintes diverses
    """

    def __init__(self, default='', doc=""):
        """
        :default: valeur par défaut de value
        :doc: description de la variable
        """
        self.default = default
        self.doc = doc
        self.value = default

    ## définition des accesseurs 
    def __get__(self, obj, objtype=None):
        """accès aux attributs par délégation
        - lorsqu'on accède aux attributs directement depuis la classe `Action`
          alors c'est le type qui est renvoyé
        - lorsqu'on accède aux attributs depuis une instance, c'est self.value
          qui est retourné
        """
        # Lorsqu'on y accède depuis la classe
        if obj is None:
            return self
        # lorsqu'on y accède depuis une instance
        return self.value

    def __set__(self, obj, value):
        """le type du paramètre est vérifié au moment
        de sa définition (son ajout dans la classe)
        """
        self.validate(value)
        self.value = value

    def validate(self, value):
        """validation du typage
        cette validation se fait au moment de la création d'un parametre"""
        raise NotImplementedError

    def get_data(self):
        """renvoie des données supplémentaires utiles pour la représentation
        ex : valeurs possibles pour un choix
        """
        return []


def check_signature(func, accepted):
    # FIXME : c'est plus la peine de wrapper puisque execute ne prend plus les paramètres
    def wrapper(self, params):
        for key, value in params.items():
            if key not in accepted:
                raise SignatureError('bad parameter type %s' % key)
            else:
                setattr(self,key,value)
        return func(self)
    return wrapper

class autoregistrable(type):
    """métaclasse d'enregistrement automatique des actions
    (et des catégories : ça peut resservir dans un refactoring de l'accès
    aux menus)
    """
    def __new__(mcs, name, bases, classdict):
        """Au moment de l'instanciation de la classe :
        - on récupère le nom de l'action et son instance et on l'enregistre
          dans un dictionnaire
        - on 
        """
        # récupérons le nom de l'action
        actionname = classdict.get('name')
        # récupérons son libelle
        actionlibelle = classdict.get('libelle')
        # récupérons la méthode execute 
        execute_func = classdict.get('execute')
        # liste des attributs de paramètres de la classe d'action
        accepted = [attrname for attrname, attrvalue in classdict.items()
                    if isinstance(attrvalue, TypedObject)]
        # liste d'attributs hérités de la classe Action 
        # on exclue les attributs spéciaux, qui commencent et finissent par '__'
        if len(bases) == 1:
            for attrname, attrvalue in bases[0].__dict__.items():
                if not attrname.startswith('__') and not attrname.endswith('__'):
                    if isinstance(attrvalue, TypedObject):
                        accepted.append(attrname)
        # validation des paramètres de la méthode execute
        if execute_func:
            classdict['execute'] = check_signature(execute_func, accepted)
        # creation de la classe action
        # uniquement si aucune autre action a le mme nom
        if not REGISTERED_ACTIONS.has_key(actionname):
            cls = type.__new__(mcs, name, bases, classdict)
        else:
            raise KeyError, "an action named %s exists already" % actionname
        
        if actionname:
            REGISTERED_ACTIONS[actionname] = cls
            category = classdict.get('category')
            # FIXME : le REGISTERED_CATEGORIES n'est pas encore utilisé
            if category not in REGISTERED_CATEGORIES:
                # FIXME : si la categorie a un slash, la decouper
                REGISTERED_CATEGORIES.append(category)

        # FIXME : faire une méthode de log automatique
        # (uniquement sur l'appel à execute)
        return cls

    def get_data(cls):
        data = {}
        for attrname, attrvalue in cls.__dict__.items():
            if isinstance(attrvalue, TypedObject):
                if hasattr(attrvalue,'get_data'):
                    data[attrname] = attrvalue.get_data()
        return data

    def get_params(cls):
        params = []
        for attrname, attrvalue in cls.__dict__.items():
            if isinstance(attrvalue, TypedObject):
                data = []
                if hasattr(attrvalue,'get_data'):
                    data = attrvalue.get_data()
                params.append((attrname, attrvalue.doc, attrvalue.__class__.__name__ , attrvalue.default, data))
        return params

 
    
class Choice(TypedObject):
    """Valeur à choix multiples"""
    def __init__(self, default='', doc="", choices=()):
        super(Choice, self).__init__(default, doc)
        self.choices = choices

    def validate(self, value):
        if value not in self.choices:
            raise TypeError("value must be one of (%s)"
                            % (', '.join( repr(c) for c in self.choices )))

    def get_data(self):
        return self.choices

class Boolean(Choice):
    """Type de donnée booléen"""
    def __init__(self, default='', doc=""):
        super(Boolean, self).__init__(default, doc, ('true', 'false'))

class Dict(TypedObject):
    """ Type de données dico """
    def __init__(self, default='', doc='', keys=[]):
        super(Dict, self).__init__(default, doc)
        self.keys = keys

    def validate(self, value):
        for key in value.keys():
            if key not in self.keys:
                raise TypeError("value shall not have the key (%s)"%key)  

class Text(TypedObject):
    """Type de données texte"""
    def __init__(self, default='', doc="", forbidden=[]):
        super(Text, self).__init__(default, doc)
        self.forbidden = forbidden

    def validate(self, value):
        try:
            assert type(value) == str
            if self.forbidden != []:
                for text in self.forbidden:
                    assert text not in value
        except:
            raise TypeError("value must be text")

    def get_data(self):
        return self.forbidden

class Path(Text):
    """Type de données chemin de fichier"""
    def __init__(self, default='', doc="", forbidden=[]):
        super(Path, self).__init__(default, doc, forbidden)

    def validate(self, value):
        super(Path, self).validate(value)
        assert os.path.exists(value)

# FIXME à tester (c'est pas bien d'avoir ecrit ça avant d'écrire un test :(
class DynamicChoice(TypedObject):
    """typage permettant d'appliquer une fonction de validation
    dynamiquement"""
    
    def __init__(self, default='', doc="", choicefunc=None):
        """:choicefunc: fonction de validation"""
        super(DynamicChoice, self).__init__(default, doc)
        self.choicefunc = choicefunc

    def validate(self, value):
        choices = self.choicefunc()
        if value not in choices:
            raise TypeError("value must be one of (%s)"
                            % (', '.join( repr(c) for c in choices )))
    def get_data(self):
        return self.choicefunc()


def get_actions():
    return REGISTERED_ACTIONS.values()

def get_full_actions():
    """
    :return: l'ensemble des actions possibles (de plugins 
    comme les actions de base
    """
    actions = {}
    actions.update(REGISTERED_ACTIONS)
    return actions.values()

def get_full_registered_actions():
    actions = {}
    actions.update(REGISTERED_ACTIONS)
    return actions

# fonctions de vérification des paramètres

def valid_strict_text(data):
    assert data.isalnum()

def valid_digit(data):
    assert data.isdigit()

def valid_path(data):
    valid_text(data)
    assert os.path.exists(data)

def valid_list(data,allowed=[]):
    assert type(data) == list
    if allowed != []:
        for val in data:
            assert val in allowed

def valid_yesno(data):
    valid_text(data)
    assert data.lower() in ['oui','o','non','n','yes','y','no']

def valid_dict(data):
    assert type(data) == dict

def valid_passwd(data):
    assert type(data) == str and len(data) > 4

def valid_boolean(data):
    assert data.lower() in ['true','false','vrai','faux']

def get_action(action_name):
    return REGISTERED_ACTIONS[action_name]

class Action:
    """classe de base pour les actions"""
    __metaclass__ = autoregistrable

    url = Text(default={}, doc="url de l'api")

    def send_all(self, result, template, templates=[]):
        """ renvoie les données au frontend pour toute la page """
        if help_templates.has_key(self.name):
            result['help'] = {'href':tools.make_help_link(self.server_nb, 'help', help_page=self.name)}
        if 'main' not in templates: templates.append('main')

        data = {'content':result}
        data['templates'] = templates
        d = {'template':template, 'data':data}
        return 0, uni(str(d))

    def send_frag(self, result, template, templates=[]):
        """ renvoie les données au frontend pour un fragment de page """
        data = {'content':result}
        if 'main' not in templates: templates.append('main')
        data['templates'] = templates
        d = {'template':template, 'data':data}
        return 0, uni(str(d))
